@@ -1,7 +1,7 @@
-import http from 'k6/http';
-import { check, sleep } from 'k6';
+import { sleep } from 'k6';
 import { Counter, Trend } from 'k6/metrics';
 import { randomIntBetween } from 'https://jslib.k6.io/k6-utils/1.2.0/index.js';
+import { Benchmarks } from './benchmark-runners/benchmarks.js';
 
 // Custom metrics
 const runDuration = new Trend('run_duration');
@@ -13,7 +13,15 @@ const serverErrors = new Counter('server_errors');
 const missingMessageErrors = new Counter('missing_message_errors');
 const otherErrors = new Counter('other_errors');
 
-// URL of your LangGraph server
+const errorMetrics = {
+  timeout_errors: timeoutErrors,
+  connection_errors: connectionErrors,
+  missing_message_errors: missingMessageErrors,
+  other_errors: otherErrors,
+  server_errors: serverErrors,
+}
+
+// URL of your Agent Server
 const BASE_URL = __ENV.BASE_URL || 'http://localhost:9123';
 // LangSmith API key only needed with a custom server endpoint
 const LANGSMITH_API_KEY = __ENV.LANGSMITH_API_KEY;
@@ -22,7 +30,9 @@ const LANGSMITH_API_KEY = __ENV.LANGSMITH_API_KEY;
 const LOAD_SIZE = parseInt(__ENV.LOAD_SIZE || '500');
 const LEVELS = parseInt(__ENV.LEVELS || '2');
 const PLATEAU_DURATION = parseInt(__ENV.PLATEAU_DURATION || '300');
-const STATEFUL = __ENV.STATEFUL === 'true';
+const BENCHMARK_TYPE = __ENV.BENCHMARK_TYPE || 'wait_write';
+const STATEFUL = __ENV.STATEFUL === 'true'; // Should the runner be stateful if possible?
+const P95_RUN_DURATION = __ENV.P95_RUN_DURATION; // Expected P95 run duration in milliseconds
 
 // Params for the agent
 const DATA_SIZE = parseInt(__ENV.DATA_SIZE || '1000');
@@ -44,6 +54,15 @@ const p95_run_duration = {
   'single': 1500,
 }
 
+function getP95RunDuration(mode) {
+  return P95_RUN_DURATION ? parseInt(P95_RUN_DURATION) : p95_run_duration[mode];
+}
+
+function getSuccessfulRunsThreshold(mode) {
+  // Number of expected successful runs per user worst case during plateau * max number of users * 2 cause that feels about right
+  return (PLATEAU_DURATION / (getP95RunDuration(mode) / 1000)) * LOAD_SIZE * LEVELS * 2;
+}
+
 // Test configuration
 export let options = {
   scenarios: {
@@ -55,100 +74,59 @@ export let options = {
     },
   },
   thresholds: {
-    'run_duration': [`p(95)<${p95_run_duration[MODE]}`],
-    'successful_runs': [`count>${(PLATEAU_DURATION / (p95_run_duration[MODE] / 1000)) * LOAD_SIZE * LEVELS * 2}`],  // Number of expected successful runs per user worst case during plateau * max number of users * 2 cause that feels about right
+    'run_duration': [`p(95)<${getP95RunDuration(MODE)}`],
+    'successful_runs': [`count>${getSuccessfulRunsThreshold(MODE)}`],
     'http_req_failed': ['rate<0.01'],   // Error rate should be less than 1%
   },
 };
 
+const runner = Benchmarks.getRunner(BENCHMARK_TYPE);
+
+const benchmarkGraphOptions = {
+  graph_id: "benchmark",
+  input: {
+    data_size: DATA_SIZE,
+    delay: DELAY,
+    expand: EXPAND,
+    mode: MODE,
+  },
+  stateful: STATEFUL,
+}
+
 // Main test function
 export default function() {
   const startTime = new Date().getTime();
-  let response;
 
+  // Prepare the request payload
+  const headers = { 'Content-Type': 'application/json' };
+  if (LANGSMITH_API_KEY) {
+    headers['x-api-key'] = LANGSMITH_API_KEY;
+  }
+  const requestParams = {
+    headers,
+    timeout: '120s',  // k6 request timeout slightly longer than the server timeout
+  };
+
+  let result;
   try {
-    // Prepare the request payload
-    const headers = { 'Content-Type': 'application/json' };
-    if (LANGSMITH_API_KEY) {
-      headers['x-api-key'] = LANGSMITH_API_KEY;
-    }
-
-    // Create a payload with the LangGraph agent configuration
-    const payload = JSON.stringify({
-        assistant_id: "benchmark",
-        input: {
-          data_size: DATA_SIZE,
-          delay: DELAY,
-          expand: EXPAND,
-          mode: MODE,
-        },
-        config: {
-          recursion_limit: EXPAND + 2,
-        },
-    });
-
-    // If the request is stateful, create a thread first and use it in the url
-    let url = `${BASE_URL}/runs/wait`;
-    if (STATEFUL) {
-      const thread = http.post(`${BASE_URL}/threads`, payload, {
-        headers,
-        timeout: '120s'  // k6 request timeout slightly longer than the server timeout
-      });
-      const threadId = thread.json().thread_id;
-      url = `${BASE_URL}/threads/${threadId}/runs/wait`;
-    }
-
-    // Make a single request to the wait endpoint
-    response = http.post(url, payload, {
-      headers,
-      timeout: '120s'  // k6 request timeout slightly longer than the server timeout
-    });
-
-    // Don't include verification in the duration of the request
-    const duration = new Date().getTime() - startTime;
-
-    // Check the response
-    const expected_length = MODE === 'single' ? 1 : EXPAND + 1;
-    let success = false;
-    try {
-      success = check(response, {
-        'Run completed successfully': (r) => r.status === 200,
-        'Response contains expected number of messages': (r) => JSON.parse(r.body)?.messages?.length === expected_length,
-      });
-    } catch (error) {
-      console.log(`Error checking response: ${error}`);
-    }
-
-
-    if (success) {
-      // Record success metrics
-      runDuration.add(duration);
-      successfulRuns.add(1);
-    } else {
-      // Handle failure
-      failedRuns.add(1);
-
-      // Classify error based on status code or response
-      if (response.status >= 500) {
-        serverErrors.add(1);
-        console.log(`Server error: ${response.status}`);
-      } else if (response.status === 408 || response.error?.includes('timeout')) {
-        timeoutErrors.add(1);
-        console.log(`Timeout error: ${response.error}`);
-      } else if (response.status === 200 && response.body?.messages?.length !== expected_length) {
-        missingMessageErrors.add(1);
-        console.log(`Missing message error: Status ${response.status}, ${JSON.stringify(response.body)}, ${response.headers?.['Content-Location']}`);
-      } else {
-        otherErrors.add(1);
-        console.log(`Other error: Status ${response.status}, ${JSON.stringify(response.body)}`);
-      }
-    }
+    result = runner.run(BASE_URL, requestParams, benchmarkGraphOptions);
   } catch (error) {
-    // Handle truly unexpected errors
     failedRuns.add(1);
     otherErrors.add(1);
-    console.log(response);
-    console.log(`Unexpected error: ${error.message}, Response Body: ${response?.body}`);
+    console.log(`Unknown error running benchmark: ${error.message}`);
+  }
+
+  // Don't include verification in the duration of the request
+  const duration = new Date().getTime() - startTime;
+
+  let success = runner.validate(result, errorMetrics, benchmarkGraphOptions);
+
+  if (success) {
+    runDuration.add(duration);
+    successfulRuns.add(1);
+  } else {
+    // Don't log the duration for failed runs
+    failedRuns.add(1);
   }
 
   // Add a small random sleep between iterations to prevent thundering herd
@@ -161,6 +139,7 @@ export function setup() {
   console.log(`Running on pod: ${__ENV.POD_NAME || 'local'}`);
   console.log(`Running with the following ramp config: load size ${LOAD_SIZE}, levels ${LEVELS}, plateau duration ${PLATEAU_DURATION}, stateful ${STATEFUL}`);
   console.log(`Running with the following agent config: data size ${DATA_SIZE}, delay ${DELAY}, expand ${EXPAND}, mode ${MODE}`);
+  console.log(`Running with the following thresholds: p95 run duration ${getP95RunDuration(MODE)}ms, successful runs threshold ${getSuccessfulRunsThreshold(MODE)}, error rate < 1%`);
 
   return { startTime: new Date().toISOString().replace(/:/g, '-').replace(/\..+/, '') };
 }
