@@ -8,8 +8,9 @@ import structlog
 from starlette.applications import Starlette
 from starlette.requests import Request
 from starlette.responses import HTMLResponse, JSONResponse, Response
-from starlette.routing import BaseRoute, Mount, Route
+from starlette.routing import BaseRoute, Route
 
+from langgraph_api.api.a2a import a2a_routes
 from langgraph_api.api.assistants import assistants_routes
 from langgraph_api.api.mcp import mcp_routes
 from langgraph_api.api.meta import meta_info, meta_metrics
@@ -52,14 +53,17 @@ async def docs(request: Request):
     return HTMLResponse(DOCS_HTML.format(mount_prefix=MOUNT_PREFIX or ""))
 
 
-meta_routes: list[BaseRoute] = [
-    Route("/ok", ok, methods=["GET"]),
-    Route("/openapi.json", openapi, methods=["GET"]),
-    Route("/docs", docs, methods=["GET"]),
+shadowable_meta_routes: list[BaseRoute] = [
     Route("/info", meta_info, methods=["GET"]),
+]
+unshadowable_meta_routes: list[BaseRoute] = [
+    Route("/ok", ok, methods=["GET"]),
     Route("/metrics", meta_metrics, methods=["GET"]),
+    Route("/docs", docs, methods=["GET"]),
+    Route("/openapi.json", openapi, methods=["GET"]),
 ]
 
+middleware_for_protected_routes = [auth_middleware]
 protected_routes: list[BaseRoute] = []
 
 if HTTP_CONFIG:
@@ -75,6 +79,8 @@ if HTTP_CONFIG:
         protected_routes.extend(ui_routes)
     if not HTTP_CONFIG.get("disable_mcp"):
         protected_routes.extend(mcp_routes)
+    if not HTTP_CONFIG.get("disable_a2a"):
+        protected_routes.extend(a2a_routes)
 else:
     protected_routes.extend(assistants_routes)
     protected_routes.extend(runs_routes)
@@ -82,9 +88,7 @@ else:
     protected_routes.extend(store_routes)
     protected_routes.extend(ui_routes)
     protected_routes.extend(mcp_routes)
-
-routes: list[BaseRoute] = []
-user_router = None
+    protected_routes.extend(a2a_routes)
 
 
 def load_custom_app(app_import: str) -> Starlette | None:
@@ -126,30 +130,18 @@ def load_custom_app(app_import: str) -> Starlette | None:
     return user_router
 
 
+user_router: Starlette | None = None
 if HTTP_CONFIG:
-    if router_import := HTTP_CONFIG.get("app"):
-        user_router = load_custom_app(router_import)
-    if not HTTP_CONFIG.get("disable_meta"):
-        routes.extend(meta_routes)
-    else:
-        # Otherwise the deployment will never be considered healthy
-        routes.append(
+    if HTTP_CONFIG.get("disable_meta"):
+        shadowable_meta_routes = [
             Route(
                 "/ok", functools.partial(ok, disabled=True), methods=["GET"], name="ok"
             )
-        )
-    if protected_routes:
-        routes.append(
-            Mount(
-                "/",
-                middleware=[auth_middleware],
-                routes=protected_routes,
-            ),
-        )
+        ]
+        unshadowable_meta_routes = []
 
-else:
-    routes.extend(meta_routes)
-    routes.append(Mount("/", middleware=[auth_middleware], routes=protected_routes))
+    if router_import := HTTP_CONFIG.get("app"):
+        user_router = load_custom_app(router_import)
 
 
 if "inmem" in MIGRATIONS_PATH:
@@ -162,4 +154,6 @@ if "inmem" in MIGRATIONS_PATH:
             await asyncio.to_thread(conn.clear)
         return JSONResponse({"ok": True})
 
-    routes.insert(0, Route("/internal/truncate", truncate, methods=["POST"]))
+    unshadowable_meta_routes.insert(
+        0, Route("/internal/truncate", truncate, methods=["POST"])
+    )
