@@ -1035,7 +1035,6 @@ async def handle_tasks_get(
         assistant_id = run_info.get("assistant_id")
         if assistant_id:
             try:
-                # Verify that the assistant exists
                 await _get_assistant(assistant_id, request.headers)
             except ValueError as e:
                 return {
@@ -1053,7 +1052,9 @@ async def handle_tasks_get(
             a2a_state = "working"
         elif lg_status == "success":
             a2a_state = "completed"
-        elif lg_status in ["error", "timeout", "interrupted"]:
+        elif lg_status == "interrupted":
+            a2a_state = "input-required"
+        elif lg_status in ["error", "timeout"]:
             a2a_state = "failed"
         else:
             a2a_state = "submitted"
