@@ -1,6 +1,8 @@
+from functools import partial
 from typing import Any
 from uuid import uuid4
 
+import jsonschema_rs
 import structlog
 
 # TODO: Remove dependency on langchain-core here.
@@ -14,7 +16,7 @@ from starlette.routing import BaseRoute
 from langgraph_api import store as api_store
 from langgraph_api.feature_flags import FF_USE_CORE_API, USE_RUNTIME_CONTEXT_API
 from langgraph_api.graph import get_assistant_id, get_graph
-from langgraph_api.grpc_ops.ops import Assistants as GrpcAssistants
+from langgraph_api.grpc.ops import Assistants as GrpcAssistants
 from langgraph_api.js.base import BaseRemotePregel
 from langgraph_api.route import ApiRequest, ApiResponse, ApiRoute
 from langgraph_api.schema import ASSISTANT_FIELDS
@@ -33,9 +35,10 @@ from langgraph_api.validation import (
     AssistantSearchRequest,
     AssistantVersionChange,
     AssistantVersionsSearchRequest,
+    ConfigValidator,
 )
 from langgraph_runtime.checkpoint import Checkpointer
-from langgraph_runtime.database import connect
+from langgraph_runtime.database import connect as base_connect
 from langgraph_runtime.ops import Assistants
 from langgraph_runtime.retry import retry_db
 
@@ -43,6 +46,8 @@ logger = structlog.stdlib.get_logger(__name__)
 
 CrudAssistants = GrpcAssistants if FF_USE_CORE_API else Assistants
 
+connect = partial(base_connect, supports_core_api=FF_USE_CORE_API)
+
 EXCLUDED_CONFIG_SCHEMA = (
     "__pregel_checkpointer",
     "__pregel_store",
@@ -165,6 +170,12 @@ async def create_assistant(request: ApiRequest) -> ApiResponse:
     payload = await request.json(AssistantCreate)
     if assistant_id := payload.get("assistant_id"):
         validate_uuid(assistant_id, "Invalid assistant ID: must be a UUID")
+    config = payload.get("config")
+    if config:
+        try:
+            ConfigValidator.validate(config)
+        except jsonschema_rs.ValidationError as e:
+            raise HTTPException(status_code=422, detail=str(e)) from e
     async with connect() as conn:
         assistant = await CrudAssistants.put(
             conn,
@@ -189,6 +200,12 @@ async def search_assistants(
     payload = await request.json(AssistantSearchRequest)
     select = validate_select_columns(payload.get("select") or None, ASSISTANT_FIELDS)
     offset = int(payload.get("offset") or 0)
+    config = payload.get("config")
+    if config:
+        try:
+            ConfigValidator.validate(config)
+        except jsonschema_rs.ValidationError as e:
+            raise HTTPException(status_code=422, detail=str(e)) from e
     async with connect() as conn:
         assistants_iter, next_offset = await CrudAssistants.search(
             conn,
@@ -241,7 +258,7 @@ async def get_assistant_graph(
     assistant_id = get_assistant_id(str(request.path_params["assistant_id"]))
     validate_uuid(assistant_id, "Invalid assistant ID: must be a UUID")
     async with connect() as conn:
-        assistant_ = await Assistants.get(conn, assistant_id)
+        assistant_ = await CrudAssistants.get(conn, assistant_id)
         assistant = await fetchone(assistant_)
     config = json_loads(assistant["config"])
     configurable = config.setdefault("configurable", {})
@@ -298,43 +315,44 @@ async def get_assistant_subgraphs(
     assistant_id = request.path_params["assistant_id"]
     validate_uuid(assistant_id, "Invalid assistant ID: must be a UUID")
     async with connect() as conn:
-        assistant_ = await Assistants.get(conn, assistant_id)
+        assistant_ = await CrudAssistants.get(conn, assistant_id)
         assistant = await fetchone(assistant_)
-        config = json_loads(assistant["config"])
-        configurable = config.setdefault("configurable", {})
-        configurable.update(get_configurable_headers(request.headers))
-        async with get_graph(
-            assistant["graph_id"],
-            config,
-            checkpointer=Checkpointer(),
-            store=(await api_store.get_store()),
-        ) as graph:
-            namespace = request.path_params.get("namespace")
 
-            if isinstance(graph, BaseRemotePregel):
-                return ApiResponse(
-                    await graph.fetch_subgraphs(
+    config = json_loads(assistant["config"])
+    configurable = config.setdefault("configurable", {})
+    configurable.update(get_configurable_headers(request.headers))
+    async with get_graph(
+        assistant["graph_id"],
+        config,
+        checkpointer=Checkpointer(),
+        store=(await api_store.get_store()),
+    ) as graph:
+        namespace = request.path_params.get("namespace")
+
+        if isinstance(graph, BaseRemotePregel):
+            return ApiResponse(
+                await graph.fetch_subgraphs(
+                    namespace=namespace,
+                    recurse=request.query_params.get("recurse", "False")
+                    in ("true", "True"),
+                )
+            )
+
+        try:
+            return ApiResponse(
+                {
+                    ns: _graph_schemas(subgraph)
+                    async for ns, subgraph in graph.aget_subgraphs(
                         namespace=namespace,
                         recurse=request.query_params.get("recurse", "False")
                         in ("true", "True"),
                     )
-                )
-
-            try:
-                return ApiResponse(
-                    {
-                        ns: _graph_schemas(subgraph)
-                        async for ns, subgraph in graph.aget_subgraphs(
-                            namespace=namespace,
-                            recurse=request.query_params.get("recurse", "False")
-                            in ("true", "True"),
-                        )
-                    }
-                )
-            except NotImplementedError:
-                raise HTTPException(
-                    422, detail="The graph does not support visualization"
-                ) from None
+                }
+            )
+        except NotImplementedError:
+            raise HTTPException(
+                422, detail="The graph does not support visualization"
+            ) from None
 
 
 @retry_db
@@ -345,40 +363,40 @@ async def get_assistant_schemas(
     assistant_id = request.path_params["assistant_id"]
     validate_uuid(assistant_id, "Invalid assistant ID: must be a UUID")
     async with connect() as conn:
-        assistant_ = await Assistants.get(conn, assistant_id)
-        # TODO Implementa  cache so we can de-dent and release this connection.
+        assistant_ = await CrudAssistants.get(conn, assistant_id)
         assistant = await fetchone(assistant_)
-        config = json_loads(assistant["config"])
-        configurable = config.setdefault("configurable", {})
-        configurable.update(get_configurable_headers(request.headers))
-        async with get_graph(
-            assistant["graph_id"],
-            config,
-            checkpointer=Checkpointer(),
-            store=(await api_store.get_store()),
-        ) as graph:
-            if isinstance(graph, BaseRemotePregel):
-                schemas = await graph.fetch_state_schema()
-                return ApiResponse(
-                    {
-                        "graph_id": assistant["graph_id"],
-                        "input_schema": schemas.get("input"),
-                        "output_schema": schemas.get("output"),
-                        "state_schema": schemas.get("state"),
-                        "config_schema": schemas.get("config"),
-                        "context_schema": schemas.get("context"),
-                    }
-                )
-
-            schemas = _graph_schemas(graph)
 
+    config = json_loads(assistant["config"])
+    configurable = config.setdefault("configurable", {})
+    configurable.update(get_configurable_headers(request.headers))
+    async with get_graph(
+        assistant["graph_id"],
+        config,
+        checkpointer=Checkpointer(),
+        store=(await api_store.get_store()),
+    ) as graph:
+        if isinstance(graph, BaseRemotePregel):
+            schemas = await graph.fetch_state_schema()
             return ApiResponse(
                 {
                     "graph_id": assistant["graph_id"],
-                    **schemas,
+                    "input_schema": schemas.get("input"),
+                    "output_schema": schemas.get("output"),
+                    "state_schema": schemas.get("state"),
+                    "config_schema": schemas.get("config"),
+                    "context_schema": schemas.get("context"),
                 }
             )
 
+        schemas = _graph_schemas(graph)
+
+        return ApiResponse(
+            {
+                "graph_id": assistant["graph_id"],
+                **schemas,
+            }
+        )
+
 
 @retry_db
 async def patch_assistant(
@@ -388,6 +406,12 @@ async def patch_assistant(
     assistant_id = request.path_params["assistant_id"]
     validate_uuid(assistant_id, "Invalid assistant ID: must be a UUID")
     payload = await request.json(AssistantPatch)
+    config = payload.get("config")
+    if config:
+        try:
+            ConfigValidator.validate(config)
+        except jsonschema_rs.ValidationError as e:
+            raise HTTPException(status_code=422, detail=str(e)) from e
     async with connect() as conn:
         assistant = await CrudAssistants.patch(
             conn,
