@@ -24,7 +24,8 @@ async def meta_info(request: ApiRequest):
             "flags": {
                 "assistants": True,
                 "crons": plus and config.FF_CRONS_ENABLED,
-                "langsmith": bool(config.LANGSMITH_API_KEY) and bool(config.TRACING),
+                "langsmith": bool(config.LANGSMITH_CONTROL_PLANE_API_KEY)
+                and bool(config.TRACING),
                 "langsmith_tracing_replicas": True,
             },
             "host": {
@@ -85,6 +86,12 @@ async def meta_metrics(request: ApiRequest):
                         "# HELP lg_api_num_running_runs The number of runs currently running.",
                         "# TYPE lg_api_num_running_runs gauge",
                         f'lg_api_num_running_runs{{project_id="{metadata.PROJECT_ID}", revision_id="{metadata.HOST_REVISION_ID}"}} {queue_stats["n_running"]}',
+                        "# HELP lg_api_pending_runs_wait_time_max The maximum time a run has been pending, in seconds.",
+                        "# TYPE lg_api_pending_runs_wait_time_max gauge",
+                        f'lg_api_pending_runs_wait_time_max{{project_id="{metadata.PROJECT_ID}", revision_id="{metadata.HOST_REVISION_ID}"}} {queue_stats.get("pending_runs_wait_time_max_secs") or 0}',
+                        "# HELP lg_api_pending_runs_wait_time_med The median pending wait time across runs, in seconds.",
+                        "# TYPE lg_api_pending_runs_wait_time_med gauge",
+                        f'lg_api_pending_runs_wait_time_med{{project_id="{metadata.PROJECT_ID}", revision_id="{metadata.HOST_REVISION_ID}"}} {queue_stats.get("pending_runs_wait_time_med_secs") or 0}',
                     ]
                 )
         except Exception as e:
