@@ -1,7 +1,7 @@
 import asyncio
-from collections.abc import AsyncIterator
+from collections.abc import AsyncIterator, Awaitable, Callable
 from typing import Literal, cast
-from uuid import uuid4
+from uuid import UUID, uuid4
 
 import orjson
 import structlog
@@ -29,15 +29,120 @@ from langgraph_api.validation import (
     RunCreateStateful,
     RunCreateStateless,
     RunsCancel,
+    ThreadCronCreate,
 )
 from langgraph_license.validation import plus_features_enabled
 from langgraph_runtime.database import connect
-from langgraph_runtime.ops import Crons, Runs, Threads
+from langgraph_runtime.ops import Crons, Runs, StreamHandler, Threads
 from langgraph_runtime.retry import retry_db
 
 logger = structlog.stdlib.get_logger(__name__)
 
 
+_RunResultFallback = Callable[[], Awaitable[bytes]]
+
+
+def _thread_values_fallback(thread_id: UUID) -> _RunResultFallback:
+    async def fetch_thread_values() -> bytes:
+        async with connect() as conn:
+            thread_iter = await Threads.get(conn, thread_id)
+            try:
+                thread = await anext(thread_iter)
+                if thread["status"] == "error":
+                    return orjson.dumps({"__error__": orjson.Fragment(thread["error"])})
+                if thread["status"] == "interrupted":
+                    # Get an interrupt for the thread. There is the case where there are multiple interrupts for the same run and we may not show the same
+                    # interrupt, but we'll always show one. Long term we should show all of them.
+                    try:
+                        if isinstance(thread["interrupts"], dict):
+                            # Handle in memory format
+                            interrupt_map = thread["interrupts"]
+                        else:
+                            interrupt_map = orjson.loads(thread["interrupts"].buf)
+                        interrupt = [next(iter(interrupt_map.values()))[0]]
+                        return orjson.dumps({"__interrupt__": interrupt})
+                    except Exception:
+                        # No interrupt, but status is interrupted from a before/after block. Default back to values.
+                        pass
+                return cast(bytes, thread["values"])
+            except StopAsyncIteration:
+                await logger.awarning(
+                    f"No checkpoint found for thread {thread_id}",
+                    thread_id=thread_id,
+                )
+                return b"{}"
+
+    return fetch_thread_values
+
+
+def _run_result_body(
+    *,
+    run_id: UUID,
+    thread_id: UUID,
+    sub: StreamHandler,
+    cancel_on_disconnect: bool = False,
+    ignore_404: bool = False,
+    fallback: _RunResultFallback | None = None,
+    cancel_message: str | None = None,
+) -> Callable[[], AsyncIterator[bytes]]:
+    last_chunk = ValueEvent()
+
+    async def consume() -> None:
+        vchunk: bytes | None = None
+        try:
+            async for mode, chunk, _ in Runs.Stream.join(
+                run_id,
+                stream_channel=sub,
+                cancel_on_disconnect=cancel_on_disconnect,
+                thread_id=thread_id,
+                ignore_404=ignore_404,
+            ):
+                if (
+                    mode == b"values"
+                    or mode == b"updates"
+                    and b"__interrupt__" in chunk
+                ):
+                    vchunk = chunk
+                elif mode == b"error":
+                    vchunk = orjson.dumps({"__error__": orjson.Fragment(chunk)})
+            if vchunk is not None:
+                last_chunk.set(vchunk)
+            elif fallback is not None:
+                last_chunk.set(await fallback())
+            else:
+                last_chunk.set(b"{}")
+        finally:
+            # Make sure to always clean up the pubsub
+            await sub.__aexit__(None, None, None)
+
+    # keep the connection open by sending whitespace every 5 seconds
+    # leading whitespace will be ignored by json parsers
+    async def body() -> AsyncIterator[bytes]:
+        try:
+            stream = asyncio.create_task(consume())
+            while True:
+                try:
+                    if stream.done():
+                        # raise stream exception if any
+                        stream.result()
+                    yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
+                    break
+                except TimeoutError:
+                    yield b"\n"
+                except asyncio.CancelledError:
+                    if cancel_message is not None:
+                        stream.cancel(cancel_message)
+                    else:
+                        stream.cancel()
+                    await stream
+                    raise
+        finally:
+            # Make sure to always clean up the pubsub
+            await sub.__aexit__(None, None, None)
+
+    return body
+
+
 @retry_db
 async def create_run(request: ApiRequest):
     """Create a run."""
@@ -104,7 +209,9 @@ async def stream_run(
     payload = await request.json(RunCreateStateful)
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
-    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
+
+    sub = await Runs.Stream.subscribe(run_id, thread_id)
+    try:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -114,20 +221,32 @@ async def stream_run(
                 run_id=run_id,
                 request_start_time=request.scope.get("request_start_time_ms"),
             )
+    except Exception:
+        # Clean up the pubsub on errors
+        await sub.__aexit__(None, None, None)
+        raise
 
-        return EventSourceResponse(
-            Runs.Stream.join(
+    async def body():
+        try:
+            async for event, message, stream_id in Runs.Stream.join(
                 run["run_id"],
                 thread_id=thread_id,
                 cancel_on_disconnect=on_disconnect == "cancel",
                 stream_channel=sub,
                 last_event_id=None,
-            ),
-            headers={
-                "Location": f"/threads/{thread_id}/runs/{run['run_id']}/stream",
-                "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
-            },
-        )
+            ):
+                yield event, message, stream_id
+        finally:
+            # Make sure to always clean up the pubsub
+            await sub.__aexit__(None, None, None)
+
+    return EventSourceResponse(
+        body(),
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run['run_id']}/stream",
+            "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
+        },
+    )
 
 
 async def stream_run_stateless(
@@ -139,7 +258,9 @@ async def stream_run_stateless(
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
     thread_id = uuid4()
-    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
+
+    sub = await Runs.Stream.subscribe(run_id, thread_id)
+    try:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -150,21 +271,33 @@ async def stream_run_stateless(
                 request_start_time=request.scope.get("request_start_time_ms"),
                 temporary=True,
             )
+    except Exception:
+        # Clean up the pubsub on errors
+        await sub.__aexit__(None, None, None)
+        raise
 
-        return EventSourceResponse(
-            Runs.Stream.join(
+    async def body():
+        try:
+            async for event, message, stream_id in Runs.Stream.join(
                 run["run_id"],
                 thread_id=run["thread_id"],
                 ignore_404=True,
                 cancel_on_disconnect=on_disconnect == "cancel",
                 stream_channel=sub,
                 last_event_id=None,
-            ),
-            headers={
-                "Location": f"/runs/{run['run_id']}/stream",
-                "Content-Location": f"/runs/{run['run_id']}",
-            },
-        )
+            ):
+                yield event, message, stream_id
+        finally:
+            # Make sure to always clean up the pubsub
+            await sub.__aexit__(None, None, None)
+
+    return EventSourceResponse(
+        body(),
+        headers={
+            "Location": f"/runs/{run['run_id']}/stream",
+            "Content-Location": f"/runs/{run['run_id']}",
+        },
+    )
 
 
 @retry_db
@@ -174,7 +307,8 @@ async def wait_run(request: ApiRequest):
     payload = await request.json(RunCreateStateful)
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
-    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
+    sub = await Runs.Stream.subscribe(run_id, thread_id)
+    try:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -184,66 +318,27 @@ async def wait_run(request: ApiRequest):
                 run_id=run_id,
                 request_start_time=request.scope.get("request_start_time_ms"),
             )
+    except Exception:
+        # Clean up the pubsub on errors
+        await sub.__aexit__(None, None, None)
+        raise
 
-        last_chunk = ValueEvent()
+    body = _run_result_body(
+        run_id=run["run_id"],
+        thread_id=run["thread_id"],
+        sub=sub,
+        cancel_on_disconnect=on_disconnect == "cancel",
+        fallback=_thread_values_fallback(thread_id),
+    )
 
-        async def consume():
-            vchunk: bytes | None = None
-            async for mode, chunk, _ in Runs.Stream.join(
-                run["run_id"],
-                thread_id=run["thread_id"],
-                stream_channel=sub,
-                cancel_on_disconnect=on_disconnect == "cancel",
-            ):
-                if (
-                    mode == b"values"
-                    or mode == b"updates"
-                    and b"__interrupt__" in chunk
-                ):
-                    vchunk = chunk
-                elif mode == b"error":
-                    vchunk = orjson.dumps({"__error__": orjson.Fragment(chunk)})
-            if vchunk is not None:
-                last_chunk.set(vchunk)
-            else:
-                async with connect() as conn:
-                    thread_iter = await Threads.get(conn, thread_id)
-                    try:
-                        thread = await anext(thread_iter)
-                        last_chunk.set(thread["values"])
-                    except StopAsyncIteration:
-                        await logger.awarning(
-                            f"No checkpoint found for thread {thread_id}",
-                            thread_id=thread_id,
-                        )
-                        last_chunk.set(b"{}")
-
-        # keep the connection open by sending whitespace every 5 seconds
-        # leading whitespace will be ignored by json parsers
-        async def body() -> AsyncIterator[bytes]:
-            stream = asyncio.create_task(consume())
-            while True:
-                try:
-                    if stream.done():
-                        # raise stream exception if any
-                        stream.result()
-                    yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
-                    break
-                except TimeoutError:
-                    yield b"\n"
-                except asyncio.CancelledError:
-                    stream.cancel()
-                    await stream
-                    raise
-
-        return StreamingResponse(
-            body(),
-            media_type="application/json",
-            headers={
-                "Location": f"/threads/{thread_id}/runs/{run['run_id']}/join",
-                "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
-            },
-        )
+    return StreamingResponse(
+        body(),
+        media_type="application/json",
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run['run_id']}/join",
+            "Content-Location": f"/threads/{thread_id}/runs/{run['run_id']}",
+        },
+    )
 
 
 @retry_db
@@ -254,7 +349,9 @@ async def wait_run_stateless(request: ApiRequest):
     on_disconnect = payload.get("on_disconnect", "continue")
     run_id = uuid7()
     thread_id = uuid4()
-    async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
+
+    sub = await Runs.Stream.subscribe(run_id, thread_id)
+    try:
         async with connect() as conn:
             run = await create_valid_run(
                 conn,
@@ -265,63 +362,37 @@ async def wait_run_stateless(request: ApiRequest):
                 request_start_time=request.scope.get("request_start_time_ms"),
                 temporary=True,
             )
+    except Exception:
+        # Clean up the pubsub on errors
+        await sub.__aexit__(None, None, None)
+        raise
 
-        last_chunk = ValueEvent()
-
-        async def consume():
-            vchunk: bytes | None = None
-            async for mode, chunk, _ in Runs.Stream.join(
-                run["run_id"],
-                thread_id=run["thread_id"],
-                stream_channel=sub,
-                ignore_404=True,
-                cancel_on_disconnect=on_disconnect == "cancel",
-            ):
-                if (
-                    mode == b"values"
-                    or mode == b"updates"
-                    and b"__interrupt__" in chunk
-                ):
-                    vchunk = chunk
-                elif mode == b"error":
-                    vchunk = orjson.dumps({"__error__": orjson.Fragment(chunk)})
-            if vchunk is not None:
-                last_chunk.set(vchunk)
-            else:
-                # we can't fetch the thread (it was deleted), so just return empty values
-                await logger.awarning(
-                    "No checkpoint emitted for stateless run",
-                    run_id=run["run_id"],
-                    thread_id=run["thread_id"],
-                )
-                last_chunk.set(b"{}")
-
-        # keep the connection open by sending whitespace every 5 seconds
-        # leading whitespace will be ignored by json parsers
-        async def body() -> AsyncIterator[bytes]:
-            stream = asyncio.create_task(consume())
-            while True:
-                try:
-                    if stream.done():
-                        # raise stream exception if any
-                        stream.result()
-                    yield await asyncio.wait_for(last_chunk.wait(), timeout=5)
-                    break
-                except TimeoutError:
-                    yield b"\n"
-                except asyncio.CancelledError:
-                    stream.cancel("Run stream cancelled")
-                    await stream
-                    raise
-
-        return StreamingResponse(
-            body(),
-            media_type="application/json",
-            headers={
-                "Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/join",
-                "Content-Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}",
-            },
+    async def stateless_fallback() -> bytes:
+        await logger.awarning(
+            "No checkpoint emitted for stateless run",
+            run_id=run["run_id"],
+            thread_id=run["thread_id"],
         )
+        return b"{}"
+
+    body = _run_result_body(
+        run_id=run["run_id"],
+        thread_id=run["thread_id"],
+        sub=sub,
+        cancel_on_disconnect=on_disconnect == "cancel",
+        ignore_404=True,
+        fallback=stateless_fallback,
+        cancel_message="Run stream cancelled",
+    )
+
+    return StreamingResponse(
+        body(),
+        media_type="application/json",
+        headers={
+            "Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}/join",
+            "Content-Location": f"/threads/{run['thread_id']}/runs/{run['run_id']}",
+        },
+    )
 
 
 @retry_db
@@ -383,11 +454,23 @@ async def join_run(request: ApiRequest):
     validate_uuid(thread_id, "Invalid thread ID: must be a UUID")
     validate_uuid(run_id, "Invalid run ID: must be a UUID")
 
-    return ApiResponse(
-        await Runs.join(
-            run_id,
-            thread_id=thread_id,
-        )
+    # A touch redundant, but to meet the existing signature of join, we need to throw any 404s before we enter the streaming body
+    await Runs.Stream.check_run_stream_auth(run_id, thread_id)
+    sub = await Runs.Stream.subscribe(run_id, thread_id)
+    body = _run_result_body(
+        run_id=run_id,
+        thread_id=thread_id,
+        sub=sub,
+        fallback=_thread_values_fallback(thread_id),
+    )
+
+    return StreamingResponse(
+        body(),
+        media_type="application/json",
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run_id}/join",
+            "Content-Location": f"/threads/{thread_id}/runs/{run_id}",
+        },
     )
 
 
@@ -402,14 +485,25 @@ async def join_run_stream(request: ApiRequest):
     validate_uuid(run_id, "Invalid run ID: must be a UUID")
     stream_mode = request.query_params.get("stream_mode") or []
     last_event_id = request.headers.get("last-event-id") or None
+
+    async def body():
+        async with await Runs.Stream.subscribe(run_id, thread_id) as sub:
+            async for event, message, stream_id in Runs.Stream.join(
+                run_id,
+                thread_id=thread_id,
+                cancel_on_disconnect=cancel_on_disconnect,
+                stream_channel=sub,
+                stream_mode=stream_mode,
+                last_event_id=last_event_id,
+            ):
+                yield event, message, stream_id
+
     return EventSourceResponse(
-        Runs.Stream.join(
-            run_id,
-            thread_id=thread_id,
-            cancel_on_disconnect=cancel_on_disconnect,
-            stream_mode=stream_mode,
-            last_event_id=last_event_id,
-        ),
+        body(),
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run_id}/stream",
+            "Content-Location": f"/threads/{thread_id}/runs/{run_id}",
+        },
     )
 
 
@@ -430,19 +524,36 @@ async def cancel_run(
         action_str if action_str in {"interrupt", "rollback"} else "interrupt",
     )
 
-    async with connect() as conn:
-        await Runs.cancel(
-            conn,
-            [run_id],
-            action=action,
-            thread_id=thread_id,
-        )
-    if wait:
-        await Runs.join(
-            run_id,
-            thread_id=thread_id,
-        )
-    return Response(status_code=204 if wait else 202)
+    sub = await Runs.Stream.subscribe(run_id, thread_id) if wait else None
+    try:
+        async with connect() as conn:
+            await Runs.cancel(
+                conn,
+                [run_id],
+                action=action,
+                thread_id=thread_id,
+            )
+    except Exception:
+        if sub is not None:
+            await sub.__aexit__(None, None, None)
+        raise
+    if not wait:
+        return Response(status_code=202)
+
+    body = _run_result_body(
+        run_id=run_id,
+        thread_id=thread_id,
+        sub=sub,
+    )
+
+    return StreamingResponse(
+        body(),
+        media_type="application/json",
+        headers={
+            "Location": f"/threads/{thread_id}/runs/{run_id}/join",
+            "Content-Location": f"/threads/{thread_id}/runs/{run_id}",
+        },
+    )
 
 
 @retry_db
@@ -517,6 +628,7 @@ async def create_cron(request: ApiRequest):
         cron = await Crons.put(
             conn,
             thread_id=None,
+            on_run_completed=payload.get("on_run_completed", "delete"),
             end_time=payload.get("end_time"),
             schedule=payload.get("schedule"),
             payload=payload,
@@ -529,12 +641,13 @@ async def create_thread_cron(request: ApiRequest):
     """Create a thread specific cron."""
     thread_id = request.path_params["thread_id"]
     validate_uuid(thread_id, "Invalid thread ID: must be a UUID")
-    payload = await request.json(CronCreate)
+    payload = await request.json(ThreadCronCreate)
 
     async with connect() as conn:
         cron = await Crons.put(
             conn,
             thread_id=thread_id,
+            on_run_completed=None,
             end_time=payload.get("end_time"),
             schedule=payload.get("schedule"),
             payload=payload,
