@@ -152,7 +152,7 @@ class ASGITransport(ASGITransportBase):
 
         try:
             await call_soon_in_main_loop(self.app(scope, receive, send))
-        except Exception:  # noqa: PIE-786
+        except Exception:  # noqa: BLE001
             if self.raise_app_exceptions:
                 raise
 
