@@ -1,8 +1,10 @@
 import os
+from collections.abc import Callable
 from os import environ, getenv
-from typing import Literal
+from typing import Literal, TypeVar, cast
 
 import orjson
+from pydantic import TypeAdapter
 from starlette.config import Config, undefined
 from starlette.datastructures import CommaSeparatedStrings
 from typing_extensions import TypedDict
@@ -22,9 +24,14 @@ class CorsConfig(TypedDict, total=False):
     max_age: int
 
 
-class ConfigurableHeaders(TypedDict):
+class ConfigurableHeaders(TypedDict, total=False):
     includes: list[str] | None
     excludes: list[str] | None
+    include: list[str] | None
+    exclude: list[str] | None
+
+
+MiddlewareOrders = Literal["auth_first", "middleware_first"]
 
 
 class HttpConfig(TypedDict, total=False):
@@ -52,6 +59,8 @@ class HttpConfig(TypedDict, total=False):
     """Prefix for mounted routes. E.g., "/my-deployment/api"."""
     configurable_headers: ConfigurableHeaders | None
     logging_headers: ConfigurableHeaders | None
+    enable_custom_route_auth: bool
+    middleware_order: MiddlewareOrders | None
 
 
 class ThreadTTLConfig(TypedDict, total=False):
@@ -121,6 +130,45 @@ class StoreConfig(TypedDict, total=False):
     ttl: TTLConfig
 
 
+class SerdeConfig(TypedDict, total=False):
+    """Configuration for the built-in serde, which handles checkpointing of state.
+
+    If omitted, no serde is set up (the object store will still be present, however)."""
+
+    allowed_json_modules: list[list[str]] | Literal[True] | None
+    """Optional. List of allowed python modules to de-serialize custom objects from.
+    
+    If provided, only the specified modules will be allowed to be deserialized.
+    If omitted, no modules are allowed, and the object returned will simply be a json object OR
+    a deserialized langchain object.
+    
+    Example:
+    {...
+        "serde": {
+            "allowed_json_modules": [
+                ["my_agent", "my_file", "SomeType"],
+            ]
+        }
+    }
+
+    If you set this to True, any module will be allowed to be deserialized.
+
+    Example:
+    {...
+        "serde": {
+            "allowed_json_modules": true
+        }
+    }
+    
+    """
+    pickle_fallback: bool
+    """Optional. Whether to allow pickling as a fallback for deserialization.
+    
+    If True, pickling will be allowed as a fallback for deserialization.
+    If False, pickling will not be allowed as a fallback for deserialization.
+    Defaults to True if not configured."""
+
+
 class CheckpointerConfig(TypedDict, total=False):
     """Configuration for the built-in checkpointer, which handles checkpointing of state.
 
@@ -133,6 +181,53 @@ class CheckpointerConfig(TypedDict, total=False):
     If provided, the checkpointer will apply TTL settings according to the configuration.
     If omitted, no TTL behavior is configured.
     """
+    serde: SerdeConfig | None
+    """Optional. Defines the configuration for how checkpoints are serialized."""
+
+
+class SecurityConfig(TypedDict, total=False):
+    securitySchemes: dict
+    security: list
+    # path => {method => security}
+    paths: dict[str, dict[str, list]]
+
+
+class CacheConfig(TypedDict, total=False):
+    cache_keys: list[str]
+    ttl_seconds: int
+    max_size: int
+
+
+class AuthConfig(TypedDict, total=False):
+    path: str
+    """Path to the authentication function in a Python file."""
+    disable_studio_auth: bool
+    """Whether to disable auth when connecting from the LangSmith Studio."""
+    openapi: SecurityConfig
+    """The schema to use for updating the openapi spec.
+
+    Example:
+        {
+            "securitySchemes": {
+                "OAuth2": {
+                    "type": "oauth2",
+                    "flows": {
+                        "password": {
+                            "tokenUrl": "/token",
+                            "scopes": {
+                                "me": "Read information about the current user",
+                                "items": "Access to create and manage items"
+                            }
+                        }
+                    }
+                }
+            },
+            "security": [
+                {"OAuth2": ["me"]}  # Default security requirement for all endpoints
+            ]
+        }
+    """
+    cache: CacheConfig | None
 
 
 # env
@@ -140,13 +235,22 @@ class CheckpointerConfig(TypedDict, total=False):
 env = Config()
 
 
-def _parse_json(json: str | None) -> dict | None:
+TD = TypeVar("TD")
+
+
+def _parse_json(json: str | None, schema: TypeAdapter | None = None) -> dict | None:
     if not json:
         return None
-    parsed = orjson.loads(json)
-    if not parsed:
-        return None
-    return parsed
+    parsed = schema.validate_json(json) if schema else orjson.loads(json)
+    return parsed or None
+
+
+def _parse_schema(schema: type[TD]) -> Callable[[str | None], TD | None]:
+    def composed(json: str | None) -> TD | None:
+        return cast(TD | None, _parse_json(json, schema=TypeAdapter(schema)))
+
+    composed.__name__ = schema.__name__  # This just gives a nicer error message if the user provides an incompatible value
+    return composed
 
 
 STATS_INTERVAL_SECS = env("STATS_INTERVAL_SECS", cast=int, default=60)
@@ -179,6 +283,9 @@ REDIS_URI = env("REDIS_URI", cast=str)
 REDIS_CLUSTER = env("REDIS_CLUSTER", cast=bool, default=False)
 REDIS_MAX_CONNECTIONS = env("REDIS_MAX_CONNECTIONS", cast=int, default=2000)
 REDIS_CONNECT_TIMEOUT = env("REDIS_CONNECT_TIMEOUT", cast=float, default=10.0)
+REDIS_HEALTH_CHECK_INTERVAL = env(
+    "REDIS_HEALTH_CHECK_INTERVAL", cast=float, default=10.0
+)
 REDIS_KEY_PREFIX = env("REDIS_KEY_PREFIX", cast=str, default="")
 RUN_STATS_CACHE_SECONDS = env("RUN_STATS_CACHE_SECONDS", cast=int, default=60)
 
@@ -189,17 +296,22 @@ ALLOW_PRIVATE_NETWORK = env("ALLOW_PRIVATE_NETWORK", cast=bool, default=False)
 See https://developer.chrome.com/blog/private-network-access-update-2024-03
 """
 
-HTTP_CONFIG: HttpConfig | None = env("LANGGRAPH_HTTP", cast=_parse_json, default=None)
-STORE_CONFIG: StoreConfig | None = env(
-    "LANGGRAPH_STORE", cast=_parse_json, default=None
-)
+# gRPC client pool size for persistence server.
+GRPC_CLIENT_POOL_SIZE = env("GRPC_CLIENT_POOL_SIZE", cast=int, default=5)
+
+# Minimum payload size to use the dedicated thread pool for JSON parsing.
+# (Otherwise, the payload is parsed directly in the event loop.)
+JSON_THREAD_POOL_MINIMUM_SIZE_BYTES = 100 * 1024  # 100 KB
+
+HTTP_CONFIG = env("LANGGRAPH_HTTP", cast=_parse_schema(HttpConfig), default=None)
+STORE_CONFIG = env("LANGGRAPH_STORE", cast=_parse_schema(StoreConfig), default=None)
 
 MOUNT_PREFIX: str | None = env("MOUNT_PREFIX", cast=str, default=None) or (
     HTTP_CONFIG.get("mount_prefix") if HTTP_CONFIG else None
 )
 
 CORS_ALLOW_ORIGINS = env("CORS_ALLOW_ORIGINS", cast=CommaSeparatedStrings, default="*")
-CORS_CONFIG: CorsConfig | None = env("CORS_CONFIG", cast=_parse_json, default=None) or (
+CORS_CONFIG = env("CORS_CONFIG", cast=_parse_schema(CorsConfig), default=None) or (
     HTTP_CONFIG.get("cors") if HTTP_CONFIG else None
 )
 """
@@ -277,8 +389,13 @@ def _parse_thread_ttl(value: str | None) -> ThreadTTLConfig | None:
     }
 
 
-CHECKPOINTER_CONFIG: CheckpointerConfig | None = env(
-    "LANGGRAPH_CHECKPOINTER", cast=_parse_json, default=None
+CHECKPOINTER_CONFIG = env(
+    "LANGGRAPH_CHECKPOINTER", cast=_parse_schema(CheckpointerConfig), default=None
+)
+SERDE: SerdeConfig | None = (
+    CHECKPOINTER_CONFIG["serde"]
+    if CHECKPOINTER_CONFIG and "serde" in CHECKPOINTER_CONFIG
+    else None
 )
 THREAD_TTL: ThreadTTLConfig | None = env(
     "LANGGRAPH_THREAD_TTL", cast=_parse_thread_ttl, default=None
@@ -290,8 +407,8 @@ N_JOBS_PER_WORKER = env("N_JOBS_PER_WORKER", cast=int, default=10)
 BG_JOB_TIMEOUT_SECS = env("BG_JOB_TIMEOUT_SECS", cast=float, default=3600)
 
 FF_CRONS_ENABLED = env("FF_CRONS_ENABLED", cast=bool, default=True)
-FF_RICH_THREADS = env("FF_RICH_THREADS", cast=bool, default=True)
 FF_LOG_DROPPED_EVENTS = env("FF_LOG_DROPPED_EVENTS", cast=bool, default=False)
+FF_LOG_QUERY_AND_PARAMS = env("FF_LOG_QUERY_AND_PARAMS", cast=bool, default=False)
 
 # auth
 
@@ -303,7 +420,7 @@ if LANGGRAPH_POSTGRES_EXTENSIONS not in ("standard", "lite"):
     raise ValueError(
         f"Unknown LANGGRAPH_POSTGRES_EXTENSIONS value: {LANGGRAPH_POSTGRES_EXTENSIONS}"
     )
-LANGGRAPH_AUTH = env("LANGGRAPH_AUTH", cast=_parse_json, default=None)
+LANGGRAPH_AUTH = env("LANGGRAPH_AUTH", cast=_parse_schema(AuthConfig), default=None)
 LANGSMITH_TENANT_ID = env("LANGSMITH_TENANT_ID", cast=str, default=None)
 LANGSMITH_AUTH_VERIFY_TENANT_ID = env(
     "LANGSMITH_AUTH_VERIFY_TENANT_ID",
@@ -334,11 +451,21 @@ LANGGRAPH_CLOUD_LICENSE_KEY = env("LANGGRAPH_CLOUD_LICENSE_KEY", cast=str, defau
 LANGSMITH_API_KEY = env(
     "LANGSMITH_API_KEY", cast=str, default=getenv("LANGCHAIN_API_KEY", "")
 )
+# LANGSMITH_CONTROL_PLANE_API_KEY is used for license verification and
+# submitting usage metadata to LangSmith SaaS.
+#
+# Use case: A self-hosted deployment can configure LANGSMITH_API_KEY
+# from a self-hosted LangSmith instance (i.e. trace to self-hosted
+# LangSmith) and configure LANGSMITH_CONTROL_PLANE_API_KEY from LangSmith SaaS
+# to facilitate license key verification and metadata submission.
+LANGSMITH_CONTROL_PLANE_API_KEY = env(
+    "LANGSMITH_CONTROL_PLANE_API_KEY", cast=str, default=LANGSMITH_API_KEY
+)
 
 # if langsmith api key is set, enable tracing unless explicitly disabled
 
 if (
-    LANGSMITH_API_KEY
+    LANGSMITH_CONTROL_PLANE_API_KEY
     and not getenv("LANGCHAIN_TRACING_V2")
     and not getenv("LANGCHAIN_TRACING")
     and not getenv("LANGSMITH_TRACING_V2")
@@ -353,9 +480,12 @@ TRACING = (
     or env("LANGSMITH_TRACING", cast=bool, default=None)
 )
 
-# if variant is "licensed", update to "local" if using LANGSMITH_API_KEY instead
+# if variant is "licensed", update to "local" if using LANGSMITH_CONTROL_PLANE_API_KEY instead
 
-if getenv("LANGSMITH_LANGGRAPH_API_VARIANT") == "licensed" and LANGSMITH_API_KEY:
+if (
+    getenv("LANGSMITH_LANGGRAPH_API_VARIANT") == "licensed"
+    and LANGSMITH_CONTROL_PLANE_API_KEY
+):
     environ["LANGSMITH_LANGGRAPH_API_VARIANT"] = "local"
 
 
@@ -375,15 +505,16 @@ API_VARIANT = env("LANGSMITH_LANGGRAPH_API_VARIANT", cast=str, default="")
 # UI
 UI_USE_BUNDLER = env("LANGGRAPH_UI_BUNDLER", cast=bool, default=False)
 
-SELF_HOSTED_METRICS_ENABLED = env(
-    "SELF_HOSTED_METRICS_ENABLED", cast=bool, default=False
-)
-SELF_HOSTED_METRICS_ENDPOINT = env(
-    "SELF_HOSTED_METRICS_ENDPOINT", cast=str, default=None
-)
-SELF_HOSTED_METRICS_EXPORT_INTERVAL_MS = env(
-    "SELF_HOSTED_METRICS_EXPORT_INTERVAL_MS", cast=int, default=60000
+LANGGRAPH_METRICS_ENABLED = env("LANGGRAPH_METRICS_ENABLED", cast=bool, default=False)
+LANGGRAPH_METRICS_ENDPOINT = env("LANGGRAPH_METRICS_ENDPOINT", cast=str, default=None)
+LANGGRAPH_METRICS_EXPORT_INTERVAL_MS = env(
+    "LANGGRAPH_METRICS_EXPORT_INTERVAL_MS", cast=int, default=60000
 )
+LANGGRAPH_LOGS_ENDPOINT = env("LANGGRAPH_LOGS_ENDPOINT", cast=str, default=None)
+LANGGRAPH_LOGS_ENABLED = env("LANGGRAPH_LOGS_ENABLED", cast=bool, default=False)
+
+SELF_HOSTED_OBSERVABILITY_SERVICE_NAME = "LGP_Self_Hosted"
+
 IS_QUEUE_ENTRYPOINT = False
 IS_EXECUTOR_ENTRYPOINT = False
 ref_sha = None
