@@ -23,8 +23,13 @@ async def cron_scheduler():
         try:
             async with connect() as conn:
                 async for cron in Crons.next(conn):
+                    on_run_completed = cron.get("on_run_completed")
+
                     run_payload = cron["payload"]
 
+                    if on_run_completed == "keep":
+                        run_payload.setdefault("on_completion", "keep")  # type: ignore[union-attr]
+
                     async with set_auth_ctx_for_run(
                         run_payload, user_id=cron["user_id"]
                     ):
