@@ -16,3 +16,9 @@ FF_USE_CORE_API = os.getenv("FF_USE_CORE_API", "false").lower() in (
     "1",
     "yes",
 )
+# Feature flag for using the JS native API
+FF_USE_JS_API = os.getenv("FF_USE_JS_API", "false").lower() in (
+    "true",
+    "1",
+    "yes",
+)
