@@ -21,7 +21,7 @@ from langgraph.pregel import Pregel
 from langgraph.store.base import BaseStore
 from starlette.exceptions import HTTPException
 
-from langgraph_api import config
+from langgraph_api import config as lg_api_config
 from langgraph_api.feature_flags import FF_USE_CORE_API, USE_RUNTIME_CONTEXT_API
 from langgraph_api.js.base import BaseRemotePregel, is_js_path
 from langgraph_api.schema import Config
@@ -51,7 +51,7 @@ async def register_graph(
     description: str | None = None,
 ) -> None:
     """Register a graph."""
-    from langgraph_api.grpc_ops.ops import Assistants as AssistantsGrpc
+    from langgraph_api.grpc.ops import Assistants as AssistantsGrpc
     from langgraph_runtime.database import connect
     from langgraph_runtime.ops import Assistants as AssistantsRuntime
 
@@ -87,7 +87,8 @@ async def register_graph(
                 description=description,
             )
 
-    await register_graph_db()
+    if not lg_api_config.IS_EXECUTOR_ENTRYPOINT:
+        await register_graph_db()
 
 
 @asynccontextmanager
@@ -358,7 +359,7 @@ async def collect_graphs_from_env(register: bool = False) -> None:
     py_specs = list(filterfalse(is_js_spec, specs))
 
     if js_specs:
-        if config.API_VARIANT == "local_dev":
+        if lg_api_config.API_VARIANT == "local_dev":
             raise NotImplementedError(
                 "LangGraph.JS graphs are not yet supported in local development mode. "
                 "To run your JS graphs, either use the LangGraph Studio application "
@@ -388,15 +389,15 @@ async def collect_graphs_from_env(register: bool = False) -> None:
         )
 
         if (
-            config.HTTP_CONFIG
-            and (js_app := config.HTTP_CONFIG.get("app"))
+            lg_api_config.HTTP_CONFIG
+            and (js_app := lg_api_config.HTTP_CONFIG.get("app"))
             and is_js_path(js_app.split(":")[0])
         ):
             js_bg_tasks.add(
                 asyncio.create_task(
                     run_js_http_process(
                         paths_str,
-                        config.HTTP_CONFIG or {},
+                        lg_api_config.HTTP_CONFIG or {},
                         watch="--reload" in sys.argv[1:],
                     ),
                 )
@@ -468,7 +469,7 @@ def _graph_from_spec(spec: GraphSpec) -> GraphValue:
             modspec.loader.exec_module(module)  # type: ignore[possibly-unbound-attribute]
         except ImportError as e:
             e.add_note(f"Could not import python module for graph:\n{spec}")
-            if config.API_VARIANT == "local_dev":
+            if lg_api_config.API_VARIANT == "local_dev":
                 e.add_note(
                     "This error likely means you haven't installed your project and its dependencies yet. Before running the server, install your project:\n\n"
                     "If you are using requirements.txt:\n"
@@ -528,7 +529,7 @@ def _graph_from_spec(spec: GraphSpec) -> GraphValue:
         elif isinstance(graph, Pregel):
             # We don't want to fail real deployments, but this will help folks catch unnecessary custom components
             # before they deploy
-            if config.API_VARIANT == "local_dev":
+            if lg_api_config.API_VARIANT == "local_dev":
                 has_checkpointer = isinstance(graph.checkpointer, BaseCheckpointSaver)
                 has_store = isinstance(graph.store, BaseStore)
                 if has_checkpointer or has_store:
@@ -644,7 +645,7 @@ def resolve_embeddings(index_config: dict) -> "Embeddings":
 
         except ImportError as e:
             e.add_note(f"Could not import embeddings module:\n{module_name}\n\n")
-            if config.API_VARIANT == "local_dev":
+            if lg_api_config.API_VARIANT == "local_dev":
                 e.add_note(
                     "If you're in development mode, make sure you've installed your project "
                     "and its dependencies:\n"
