@@ -70,7 +70,7 @@ class HTTPMetricsCollector:
         hist_data["count"] += 1
 
         try:
-            if config.SELF_HOSTED_METRICS_ENABLED:
+            if config.LANGGRAPH_METRICS_ENABLED:
                 from langgraph_api.self_hosted_metrics import record_http_request
 
                 record_http_request(method, route_path, status, latency_seconds)
