@@ -1,8 +1,6 @@
 # ruff: noqa: E402
 import os
 
-from langgraph_api.api.meta import METRICS_FORMATS
-
 if not (
     (disable_truststore := os.getenv("DISABLE_TRUSTSTORE"))
     and disable_truststore.lower() == "true"
@@ -23,7 +21,7 @@ import structlog
 
 from langgraph_api.utils.errors import GraphLoadError, HealthServerStartupError
 from langgraph_runtime import lifespan
-from langgraph_runtime.database import pool_stats
+from langgraph_runtime.database import healthcheck, pool_stats
 from langgraph_runtime.metrics import get_metrics
 
 logger = structlog.stdlib.get_logger(__name__)
@@ -45,10 +43,14 @@ async def health_and_metrics_server():
     from starlette.responses import JSONResponse, PlainTextResponse
     from starlette.routing import Route
 
+    from langgraph_api.api.meta import METRICS_FORMATS
+
     port = int(os.getenv("PORT", "8080"))
     host = os.getenv("LANGGRAPH_SERVER_HOST", "0.0.0.0")
 
     async def health_endpoint(request):
+        # if db or redis is not healthy, this will raise an exception
+        await healthcheck()
         return JSONResponse({"status": "ok"})
 
     async def metrics_endpoint(request: Request):
