@@ -16,7 +16,15 @@ RunStatus = Literal["pending", "running", "error", "success", "timeout", "interr
 ThreadStatus = Literal["idle", "busy", "interrupted", "error"]
 
 StreamMode = Literal[
-    "values", "messages", "updates", "events", "debug", "tasks", "checkpoints", "custom"
+    "values",
+    "messages",
+    "updates",
+    "events",
+    "debug",
+    "tasks",
+    "checkpoints",
+    "custom",
+    "messages-tuple",
 ]
 
 ThreadStreamMode = Literal["lifecycle", "run_modes", "state_update"]
@@ -50,7 +58,7 @@ class Config(TypedDict, total=False):
     """
     Runtime values for attributes previously made configurable on this Runnable,
     or sub-Runnables, through .configurable_fields() or .configurable_alternatives().
-    Check .output_schema() for a description of the attributes that have been made 
+    Check .output_schema() for a description of the attributes that have been made
     configurable.
     """
 
@@ -148,7 +156,7 @@ class ThreadState(TypedDict):
     next: Sequence[str]
     """The name of the node to execute in each task for this step."""
     checkpoint: Checkpoint
-    """The checkpoint keys. This object can be passed to the /threads and /runs 
+    """The checkpoint keys. This object can be passed to the /threads and /runs
     endpoints to resume execution or update state."""
     metadata: Fragment
     """Metadata for this state"""
@@ -220,6 +228,8 @@ class Cron(TypedDict):
     """The ID of the assistant."""
     thread_id: UUID | None
     """The ID of the thread."""
+    on_run_completed: NotRequired[Literal["delete", "keep"] | None]
+    """What to do with the thread after the run completes."""
     end_time: datetime | None
     """The end date to stop running the cron."""
     schedule: str
@@ -249,8 +259,8 @@ class ThreadUpdateResponse(TypedDict):
 class QueueStats(TypedDict):
     n_pending: int
     n_running: int
-    max_age_secs: datetime | None
-    med_age_secs: datetime | None
+    pending_runs_wait_time_max_secs: float | None
+    pending_runs_wait_time_med_secs: float | None
 
 
 # Canonical field sets for select= validation and type aliases for ops
@@ -303,6 +313,7 @@ CronSelectField = Literal[
     "cron_id",
     "assistant_id",
     "thread_id",
+    "on_run_completed",
     "end_time",
     "schedule",
     "created_at",
