@@ -26,11 +26,17 @@ from langgraph_sdk.client import configure_loopback_transports
 from starlette.applications import Starlette
 from starlette.middleware import Middleware
 from starlette.middleware.cors import CORSMiddleware
-from starlette.routing import Mount
+from starlette.routing import BaseRoute, Mount
 from starlette.types import Receive, Scope, Send
 
 import langgraph_api.config as config
-from langgraph_api.api import meta_routes, routes, user_router
+from langgraph_api.api import (
+    middleware_for_protected_routes,
+    protected_routes,
+    shadowable_meta_routes,
+    unshadowable_meta_routes,
+    user_router,
+)
 from langgraph_api.api.openapi import set_custom_spec
 from langgraph_api.errors import (
     overloaded_error_handler,
@@ -48,10 +54,10 @@ from langgraph_runtime.retry import OVERLOADED_EXCEPTIONS
 logging.captureWarnings(True)
 logger = structlog.stdlib.get_logger(__name__)
 
-middleware = []
+global_middleware = []
 
 if config.ALLOW_PRIVATE_NETWORK:
-    middleware.append(Middleware(PrivateNetworkMiddleware))
+    global_middleware.append(Middleware(PrivateNetworkMiddleware))
 
 JS_PROXY_MIDDLEWARE_ENABLED = (
     config.HTTP_CONFIG
@@ -62,9 +68,9 @@ JS_PROXY_MIDDLEWARE_ENABLED = (
 if JS_PROXY_MIDDLEWARE_ENABLED:
     from langgraph_api.js.remote import JSCustomHTTPProxyMiddleware
 
-    middleware.append(Middleware(JSCustomHTTPProxyMiddleware))
+    global_middleware.append(Middleware(JSCustomHTTPProxyMiddleware))
 
-middleware.extend(
+global_middleware.extend(
     [
         (
             Middleware(
@@ -111,7 +117,7 @@ def update_openapi_spec(app):
         schemas = SchemaGenerator(
             {
                 "openapi": "3.1.0",
-                "info": {"title": "LangGraph Platform", "version": "0.1.0"},
+                "info": {"title": "LangSmith Deployment", "version": "0.1.0"},
             }
         )
         spec = schemas.get_schema(routes=app.routes)
@@ -120,32 +126,98 @@ def update_openapi_spec(app):
         set_custom_spec(spec)
 
 
+def apply_middleware(
+    routes: list[BaseRoute], middleware: list[Middleware]
+) -> list[BaseRoute]:
+    """Applies middleware to a list of routes.
+
+    Routes are modified in place (only the `app` attribute is modified);
+    the modified routes are returned for convenience.
+    """
+    middleware_routes = []
+    for route in routes:
+        for cls, args, kwargs in reversed(middleware):
+            if hasattr(route, "app"):
+                route.app = cls(route.app, *args, **kwargs)  # type: ignore
+            else:
+                raise ValueError(f"Cannot apply middleware: route {route} has no app")
+        middleware_routes.append(route)
+    return middleware_routes
+
+
+custom_middleware = (
+    user_router.user_middleware if user_router and user_router.user_middleware else []
+)
+auth_before_custom_middleware = (
+    config.HTTP_CONFIG and config.HTTP_CONFIG.get("middleware_order") == "auth_first"
+)
+enable_auth_on_custom_routes = config.HTTP_CONFIG and config.HTTP_CONFIG.get(
+    "enable_custom_route_auth"
+)
+# Custom middleware to be applied at the route/mount level, not globally (app level).
+route_level_custom_middleware = (
+    custom_middleware if auth_before_custom_middleware else []
+)
+
+protected_mount = Mount(
+    "",
+    routes=protected_routes,
+    middleware=(
+        middleware_for_protected_routes + route_level_custom_middleware
+        if auth_before_custom_middleware
+        else route_level_custom_middleware + middleware_for_protected_routes
+    ),
+)
+
+
 if user_router:
     # Merge routes
     app = user_router
+    if auth_before_custom_middleware:
+        # Authentication middleware is only applied to protected routes--
+        # it is *not* global middleware. This means that by default,
+        # authentication middleware is necessarily applied *after* any global middleware.
+        # including custom middleware that the user might have supplied.
+        #
+        # To apply authentication middleware before custom middleware,
+        # we must rearrange things a bit:
+        #   1. Extract user-supplied routes and bundle them into a `Mount`
+        #      so that we can easily apply custom middleware to all of them at once.
+        #   2. Extract custom middleware from the user-supplied app.
+        #      Remove it globally, but apply it to each bundle of routes at the mount level.
+        #      This gives us more flexibility in ordering: we can now apply this
+        #      custom middleware before *or* after authentication middleware,
+        #      depending on the `middleware_order` config.
+        user_app = apply_middleware(
+            routes=app.routes,
+            middleware=(
+                middleware_for_protected_routes if enable_auth_on_custom_routes else []
+            )
+            + route_level_custom_middleware,
+        )
+        app.user_middleware = global_middleware
+    else:
+        user_app = (
+            apply_middleware(
+                routes=app.routes,
+                middleware=middleware_for_protected_routes,
+            )
+            if enable_auth_on_custom_routes
+            else app.routes
+        )
+        app.user_middleware = custom_middleware + global_middleware
 
-    meta_route_paths = [
-        getattr(route, "path", None) for route in meta_routes if hasattr(route, "path")
-    ]
-    custom_route_paths = [
-        route.path
-        for route in user_router.router.routes
-        if hasattr(route, "path") and route.path not in meta_route_paths
-    ]
-    logger.info(f"Custom route paths: {custom_route_paths}")
+    app.router.routes = (
+        apply_middleware(unshadowable_meta_routes, route_level_custom_middleware)
+        + user_app
+        + apply_middleware(shadowable_meta_routes, route_level_custom_middleware)
+        + [protected_mount]
+    )
 
     update_openapi_spec(app)
-    for route in routes:
-        if getattr(route, "path", None) in ("/docs", "/openapi.json"):
-            # Our handlers for these are inclusive of the custom routes and default API ones
-            # Don't let these be shadowed
-            app.router.routes.insert(0, route)
-        else:
-            # Everything else could be shadowed.
-            app.router.routes.append(route)
 
-    # Merge lifespans
-    original_lifespan = app.router.lifespan_context
+    # Merge lifespans (base + user)
+    user_lifespan = app.router.lifespan_context
     if app.router.on_startup or app.router.on_shutdown:
         raise ValueError(
             f"Cannot merge lifespans with on_startup or on_shutdown: {app.router.on_startup} {app.router.on_shutdown}"
@@ -154,17 +226,15 @@ if user_router:
     @asynccontextmanager
     async def combined_lifespan(app):
         async with lifespan(app):
-            if original_lifespan:
-                async with original_lifespan(app):
+            if user_lifespan:
+                async with user_lifespan(app):
                     yield
             else:
                 yield
 
     app.router.lifespan_context = combined_lifespan
 
-    # Merge middleware
-    app.user_middleware = (app.user_middleware or []) + middleware
-    # Merge exception handlers
+    # Merge exception handlers (base + user)
     for k, v in exception_handlers.items():
         if k not in app.exception_handlers:
             app.exception_handlers[k] = v
@@ -173,16 +243,20 @@ if user_router:
     # If the user creates a loopback client with `get_client() (no url)
     # this will update the http transport to connect to the right app
     configure_loopback_transports(app)
-
 else:
     # It's a regular starlette app
     app = Starlette(
-        routes=routes,
+        routes=apply_middleware(
+            unshadowable_meta_routes + shadowable_meta_routes,
+            route_level_custom_middleware,
+        )
+        + [protected_mount],
         lifespan=lifespan,
-        middleware=middleware,
+        middleware=global_middleware,
         exception_handlers=exception_handlers,
     )
 
+
 if config.MOUNT_PREFIX:
     prefix = config.MOUNT_PREFIX
     if not prefix.startswith("/") or prefix.endswith("/"):
