@@ -269,7 +269,7 @@ async def worker(
             elif isinstance(exception, TimeoutError):
                 status = "timeout"
                 await logger.awarning(
-                    "Background run timed out",
+                    "Background run timed out. To increase the timeout, set the BG_JOB_TIMEOUT_SECS environment variable (integer, defaults to 3600).",
                     **log_info,
                 )
                 if not temporary:
